@@ -1,5 +1,12 @@
-// no merging, this is the entire library. it is literally just enough to run the bootstrap program that prints out C constants for us,
-// we obviously need to run without any such constants ourselves...
+// Copyright 2015 The Emscripten Authors.  All rights reserved.
+// Emscripten is available under two separate licenses, the MIT license and the
+// University of Illinois/NCSA Open Source License.  Both these licenses can be
+// found in the LICENSE file.
+//
+// no merging, this is the entire library. it is literally just enough to run
+// the bootstrap program that prints out C constants for us, we obviously need
+// to run without any such constants ourselves...
+
 assert(!LibraryManager.library);
 LibraryManager.library = {
   sysconf: function(name) {
@@ -20,18 +27,18 @@ LibraryManager.library = {
     // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
     var self = _sbrk;
     if (!self.called) {
-      HEAP32[DYNAMICTOP_PTR>>2] = alignMemoryPage(HEAP32[DYNAMICTOP_PTR>>2]); // make sure we start out aligned
+      HEAP32[DYNAMICTOP_PTR>>2] = alignUp(HEAP32[DYNAMICTOP_PTR>>2], 16777216); // make sure we start out aligned
       self.called = true;
-      assert(Runtime.dynamicAlloc);
-      self.alloc = Runtime.dynamicAlloc;
-      Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
+      assert(dynamicAlloc);
+      self.alloc = dynamicAlloc;
+      dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
     }
     var ret = HEAP32[DYNAMICTOP_PTR>>2];
     if (bytes != 0) self.alloc(bytes);
     return ret;  // Previous break location.
   },
   malloc: function(x) {
-    return Runtime.dynamicAlloc(x);
+    return dynamicAlloc(x);
   },
 };
 
